class Passbox.Models.Password extends Backbone.Model
	paramRoot: 'password'
	idAttribute: "website"
	
	defaults:
		website: null
		username: null
		password: null
	
	constructor: ->
		super
		@sync = ModelSync
		
	url: 'passwords'

class Passbox.Collections.Passwords extends Backbone.Collection
	model: Passbox.Models.Password
	url: 'passwords'
	
	constructor: ->
		super
		@sync = CollectionSync
		@_decrypted = false
		
	save: (options = {}) ->
		@_decrypted = true
		ModelSync("create", {collection:this}, options)
		
	encrypt: ->
		plaintext = JSON.stringify @toJSON()
		ciphertext = sjcl.encrypt Passbox.user.get("key"), plaintext
		return ciphertext
		
	decrypt: (ciphertext) ->
		try
			plaintext = sjcl.decrypt Passbox.user.get("key"), ciphertext
			plaintext = JSON.parse plaintext
			@_decrypted = true
			return plaintext
		catch error
			console.log "Decrypt Error", error
			return false
			
	isDecrypted: ->
		@_decrypted

CollectionSync = (method, collection, options, error) ->		
	tryToDecrypt = (ciphertext) ->
		if (plaintext = collection.decrypt ciphertext)
			options.success plaintext
		else
			options.error "DecryptError"
	
	switch method
		when "read"
			if collection.cryptic 
				tryToDecrypt collection.cryptic
			else
				$.ajax({
					url: getUrl(collection)
					dataType: 'json'
					# contentType: 'text/html'
					contentType: 'application/json'
					cache: false
					success: (data) ->
						if data.payload
							collection.cryptic = data.payload
							tryToDecrypt data.payload
						else if data.status == "FileNotFound"
							options.error "FileNotFound"
						else if data.status == "NotAuthorized"
							options.error "NotAuthorized"
						else
							options.error "DropboxApiError"

					error: ->
						console.log "error"
				})
	
ModelSync = (method, model, options, error) ->
	switch method
		when "read"
			resp = (if model.id then model else model.collection)
		when "create", "update", "delete"
		
			params = #_.extend
				type:        'POST'
				dataType:    'json'
				contentType: 'text/plain'
				cache: false
			# , options
			
			collection = model.collection
			options.success model
			
			ciphertext = collection.encrypt()
			
			params.data = ciphertext
			params.url ||= getUrl(collection)
			params.processData = false
		
			return $.ajax(params)
			
	if resp
		options.success resp
	else
		options.error "Record not found"
		

getUrl = (object) ->
	if !(object and object.url) then return null
	return if _.isFunction(object.url) then object.url() else object.url