require 'dropbox_sdk'

class DbController < ApplicationController
	
	#before_filter :delay # for testing

	def authorize		
		if not params[:oauth_token] then
			authStart
		else
			authCallback
		end
	end

	def authStart
		# OAuth Start

		# OAuth Step 1: Get a request token from Dropbox
		db_session = DropboxSession.new(DROPBOX_CONFIG["APP_KEY"], DROPBOX_CONFIG["APP_SECRET"])
		begin
			db_session.get_request_token
		rescue DropboxError => e
			render :text => "Exception in OAuth Step 1: #{escape_html e}"
		
		#serialize and save this DropboxSession
		session[:request_db_session] = dbsession.serialize
		
		# OAuth Step 2: Send the user to the Dropbox website so they can authorize
		# our app. After the user authorizes our app, Dropbox will redirect them
		# to our '/oauth-callback' endpoint.

		# pass to get_authorize_url a callback url that will return the user here
		url = dbsession.get_authorize_url url_for(:action => 'authorize')

		respond_to do |format|
			format.json { render :json => {'status' => 'redirect', 'url' => url}}
			format.html { redirect_to url }
		end
	end

	def authCallback
		# Finish OAuth Step 2: the user has returned from dropbox
		serialized_session = session[:request_db_session]
		unless serialized_session
			# should not ever go here
			render :text => "Error in OAuth step 2: Couldn't find OAuth state in session"
		end
		db_session = DropboxSession.deserialize(ser)

		# OAuth Step 3: Get an access token from Dropbox.
		begin
			db_session.get_access_token
		rescue DropboxError => e
			render :text => "Exception in OAuth step 3: #{escape_html e}"
		end

		session.delete(:request_db_session)
		session[:authorized_db_session] = db_session.serialize

		respond_to do |format|
			format.json { render :json => {'success' => true } }
			format.html { redirect_to '' }
		end

		# Consider storing the authorized DropboxSession somewhere more persistent.
		# Instead of in the web session hash.
	end
	
	def logout
		
		#TODO: does this belong here?
		session.delete :authorized_db_session

		respond_to do |format|
			format.json { render :json => {'success' => true }}
			format.html { redirect_to '' }
		end
	end
 
	def push
		client = get_client()
		unless client
			render :text => "error"
		end
		
		# def put_file(to_path, file_obj, overwrite=false, parent_rev=nils)
		resp = client.put_file(DROPBOX_CONFIG["DB_FILENAME"], request.raw_post, true)
		render :json => {'success' => true }
	end 
	
	def pull
		client = get_client()
		unless client
			redirect_to(:action => 'authorize')
		end
		
		begin
			file = client.get_file(DROPBOX_CONFIG["DB_FILENAME"])
			
			# the file doesn't exist
			rescue DropboxError => err
				if err.error == "File has been deleted"
					render :text => ""
					return
					# file = "[]"
					# client.put_file(DROPBOX_CONFIG["DB_FILENAME"], file)
				else
					puts "ERROR: db_controller pull() new DropboxError discovered!"
				end
				
		end
		
		render :text => file
	end
	
	private
		def get_client
			
			# Check if user has no dropbox session...re-direct them to authorize
			return redirect_to(:action => 'authorize') unless session[:dropbox_session]
			
			dbsession = DropboxSession.deserialize(session[:dropbox_session])
			client = DropboxClient.new(dbsession, DROPBOX_CONFIG["ACCESS_TYPE"]) #raise an exception if session not authorized
			
			return client
		end

		def get_client
			if session[:dropbox_session]
				db_session = DropboxSession.deserialize(session[:dropbox_session])
				begin
					return DropboxClient.new(db_session, DROPBOX_CONFIG["ACCESS_TYPE"])
				rescue DropboxAuthError => e
					# The stored session didn't work. Fall through and start OAuth
					session[:dropbox_session].delete
			end
		end
		
		def delay
			sleep(2.0)
		end
end
