class User
	include Mongoid::Document
	include Mongoid::Timestamps

	# Standard info
	field :uid, :type => Integer
	field :email, :type => String
	field :user_info, :type => Hash

	field :access_key, :type => String
	field :access_secret, :type => String

	# Trackable
	field :last_sign_in_at, :type => DateTime
	field :current_sign_in_at, :type => DateTime
	field :last_sign_in_ip, :type => String
	field :current_sign_in_ip, :type => String
	field :sign_in_count, :type => Integer, :default => 0
	field :usage_count, :type => Integer, :default => 0

	def save_access_token!(token)
		self.access_key = token.key
		self.access_secret = token.secret
		self.save!
	end

	# Find or create from dropbox hash
	def self.find_or_create_from_hash(hash)
		uid, email = hash["uid"], hash["email"]
		
		# Find user
		user = self.where(uid: uid).first
		unless user
			user = User.new(:uid => uid)
		end

		# Update user
		user.update_attributes(:email => email, :user_info => hash)
		user.save

		# Return user
		user
	end

	# Call on every login
	def track_login!(request) 
		# Timestamps
		old_current, new_current = self.current_sign_in_at, Time.now
		self.last_sign_in_at = old_current || new_current
		self.current_sign_in_at = new_current

		# IPs
		old_current, new_current = self.current_sign_in_ip, request.remote_ip
        self.last_sign_in_ip = old_current || new_current
        self.current_sign_in_ip = new_current

        # Sign in count
		self.sign_in_count ||= 0
		self.sign_in_count += 1

		# Save
		self.save!
	end

	# Call whenever user's non-login data is modified
	def track_usage!()
		self.usage_count ||= 0
		self.usage_count += 1
		self.save!
	end
end
